
Cockpit Internationalization
============================

All human-visible strings on Cockpit's web pages should be translatable. This
document describes the entire internationalization process, which uses [GNU gettext](https://www.gnu.org/software/gettext/) and related tools as much as possible.

Marking strings
---------------
Strings that are visible to humans and which should be translated must be
marked as such on Cockpit pages, in one of two ways:

 * If the string appears in a HTML source, it must be enclosed in a tag with
   the `translate="yes"` attribute (some code also still has the deprecated
   `translatable="yes"`). Example:

   ```html
   <span translate="yes">Image</span>
   ```

 * If the string appears in JavaScript source, it must use the
   [cockpit.gettext()](https://cockpit-project.org/guide/latest/cockpit-locale.html#cockpit-locale-gettext)
   or a related function, commonly called through an alias `_()`, with double-quoted
   strings so that `xgettext` can recognize it. Example:

   ```js
   const _ = cockpit.gettext;

   var translated = _("Time");

   // in a JSX block:
   <tr>
     <th>{ _("Image") }</th>
   </tr>
   ```
   For strings that involve numbers you need to use
   [cockpit.ngettext()](https://cockpit-project.org/guide/latest/cockpit-locale.html#cockpit-locale-ngettext)
   instead:

   ```js
   var translated = cockpit.ngettext("This thing", "The things", numberOfThings);
   ```

   The [cockpit.format()](https://cockpit-project.org/guide/latest/cockpit-util.html#cockpit-format)
   function is useful for parameterized strings where the order of parameters
   might change between languages.

Translation process
-------------------

 * Translatable strings from HTML, JavaScript (as above) and C files get
   extracted with various invocations of
   [xgettext](https://linux.die.net/man/1/xgettext), see
   [po/Makefile.am](https://github.com/cockpit-project/cockpit/blob/master/po/Makefile.am).
   Running `make po/cockpit.pot` will generate a standard PO template.

 * With `make upload-pot` the PO template gets uploaded to the
   [Zanata translation platform](https://fedora.zanata.org/iteration/view/cockpit/master)
   where everybody can contribute translations to various languages.

 *  With `make download-po` Zanata's translations are downloaded to
    `po/XX.po`. This is done by
   [bots/po-refresh](https://github.com/cockpit-project/cockpit/blob/master/bots/po-refresh)
   which is invoked regularly by
   [bots/po-trigger](https://github.com/cockpit-project/cockpit/blob/master/bots/po-trigger).

 * Translations can also be done using any other tool or platform and be
   submitted via pull requests that update the `po/XX.po` files. In this case
   they should then also be uploaded to Zanata again with
   `make upload-translations`.

Using translations at runtime
-----------------------------
HTML and JavaScript don't directly support gettext po and mo files. The
translations from po files are converted to a JavaScript function by
[po/po2json](https://github.com/cockpit-project/cockpit/blob/master/po/po2json)
which inserts the actual translations into the
[po.empty.js template](https://github.com/cockpit-project/cockpit/blob/master/po/po.empty.js).
The resulting files are written to `dist/<page>/po.XX.js` from where they
can be imported by the page and used by `cockpit.gettext()`.

HTML pages which use `translate="yes"` tags need to call
[cockpit.translate()](https://cockpit-project.org/guide/latest/cockpit-locale.html#cockpit-locale-translate) which will iterate over all such tags and replace their content with the result of calling `cockpit.gettext()` on their original content.

Opening a new branch
--------------------
Occasionally we create a new upstream branch for e. g. a new stable RHEL
release. Run these steps to initialize translations for it:

 * On the [Zanata Cockpit project page](https://fedora.zanata.org/project/view/cockpit),
   open the … menu in "Versions" and create a "New Version". Give it a name
   that matches the new branch, and import existing translations from master
   (or another branch if that's more appropriate). This will take a while to
   get initialized.

 * On the new cockpit branch, change `<project-version>` in
   [po/zanata.xml](https://github.com/cockpit-project/cockpit/blob/master/po/zanata.xml)
   from "master" to the new branch name.

 * Run `make upload-pot upload-translations` to push the current translations
   from git to it. The majority should already exist through the import from
   above, but this makes sure the translations are completely in sync.
